(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('@angular/core')) :
    typeof define === 'function' && define.amd ? define('ngx-remotedata', ['exports', 'rxjs', '@angular/core'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global['ngx-remotedata'] = {}, global.rxjs, global.ng.core));
}(this, (function (exports, rxjs, core) { 'use strict';

    // ----------------------------
    //
    //  Constructor functions
    //
    // ----------------------------
    var notAsked = function () {
        return { tag: 'NotAsked' };
    };
    var inProgress = function (value) {
        return { tag: 'InProgress', value: value };
    };
    var failure = function (error, value) {
        return { tag: 'Failure', error: error, value: value };
    };
    var success = function (value) {
        return { tag: 'Success', value: value };
    };
    // ----------------------------
    //
    //  Guards
    //
    // ----------------------------
    var isNotAsked = function (value) {
        return (value !== null &&
            value !== undefined &&
            value.hasOwnProperty('tag') &&
            value.tag === 'NotAsked');
    };
    var isInProgress = function (value) {
        return (value !== null &&
            value !== undefined &&
            value.hasOwnProperty('tag') &&
            value.tag === 'InProgress');
    };
    var isFailure = function (value) {
        return (value !== null &&
            value !== undefined &&
            value.hasOwnProperty('tag') &&
            value.tag === 'Failure');
    };
    var isSuccess = function (value) {
        return (value !== null &&
            value !== undefined &&
            value.hasOwnProperty('tag') &&
            value.tag === 'Success');
    };
    var isRemoteData = function (value) {
        var hasRemoteDataTag = function (tag) { return tag === 'NotAsked' ||
            tag === 'InProgress' ||
            tag === 'Success' ||
            tag === 'Failure'; };
        return (value !== null &&
            value !== undefined &&
            value.hasOwnProperty('tag') &&
            hasRemoteDataTag(value.tag));
    };
    // ----------------------------
    //
    //  Folds
    //
    // ----------------------------
    var fold = function (onNotAsked, onInProgress, onFailure, onSuccess, rd) {
        switch (rd.tag) {
            case 'NotAsked':
                return onNotAsked();
            case 'InProgress':
                return onInProgress(rd.value);
            case 'Failure':
                return onFailure(rd.error, rd.value);
            case 'Success':
                return onSuccess(rd.value);
        }
    };
    var getOrElse = function (rd, defaultValue) {
        switch (rd.tag) {
            case 'Success':
                return rd.value;
            default:
                return defaultValue;
        }
    };
    // ----------------------------
    //
    //  Transformations
    //
    // ----------------------------
    var map = function (fn, rd) { return isSuccess(rd) ? success(fn(rd.value)) : rd; };
    var mapFailure = function (fn, rd) { return isFailure(rd) ? failure(fn(rd.error)) : rd; };
    var chain = function (fn, rd) { return isSuccess(rd) ? fn(rd.value) : rd; };
    // ----------------------------
    //
    //  RxJs specific
    //
    // ----------------------------
    /**
     * Emits only when source Observable is a Success
     * @returns Success<T>
     */
    function filterSuccess() {
        return function (source) { return new rxjs.Observable(function (subscriber) {
            source.subscribe({
                next: function (value) {
                    if (isSuccess(value)) {
                        subscriber.next(value);
                    }
                }
            });
        }); };
    }
    /**
     * Emits only when source Observable is a Failure
     * @returns Failure<E, T>
     */
    function filterFailure() {
        return function (source) { return new rxjs.Observable(function (subscriber) {
            source.subscribe({
                next: function (value) {
                    if (isFailure(value)) {
                        subscriber.next(value);
                    }
                }
            });
        }); };
    }

    var assertIsRemoteData = function (rd) {
        if (!isRemoteData(rd)) {
            throw new Error("Value \"" + rd + "\" is not a RemoteData<T> instance. Did you forget to use the async pipe? i.e: state$ | async | isInProgress");
        }
    };
    var ɵ0 = assertIsRemoteData;
    var IsNotAskedPipe = /** @class */ (function () {
        function IsNotAskedPipe() {
        }
        IsNotAskedPipe.prototype.transform = function (rd) {
            if (rd === null || rd === undefined) {
                return false;
            }
            assertIsRemoteData(rd);
            return isNotAsked(rd);
        };
        return IsNotAskedPipe;
    }());
    IsNotAskedPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'isNotAsked' },] }
    ];
    var IsInProgressPipe = /** @class */ (function () {
        function IsInProgressPipe() {
        }
        IsInProgressPipe.prototype.transform = function (rd) {
            if (rd === null || rd === undefined) {
                return false;
            }
            assertIsRemoteData(rd);
            return isInProgress(rd);
        };
        return IsInProgressPipe;
    }());
    IsInProgressPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'isInProgress' },] }
    ];
    var AnyIsInProgressPipe = /** @class */ (function () {
        function AnyIsInProgressPipe(cd) {
            this.cd = cd;
            this._latestValue = false;
            this._subscription = null;
            this._rds$ = [];
        }
        AnyIsInProgressPipe.prototype.transform = function (rds$) {
            var _this = this;
            if (rds$ === null || rds$ === undefined) {
                return false;
            }
            if (this._rds$ === rds$) {
                return this._latestValue;
            }
            else if (this._subscription) {
                this.dispose();
            }
            this._rds$ = rds$;
            this._subscription = rxjs.combineLatest(rds$).subscribe(function (rds) {
                _this._latestValue = rds.some(isInProgress);
                _this.cd.markForCheck();
            });
            return this._latestValue;
        };
        AnyIsInProgressPipe.prototype.ngOnDestroy = function () {
            this.dispose();
        };
        AnyIsInProgressPipe.prototype.dispose = function () {
            if (this._subscription) {
                this._subscription.unsubscribe();
            }
            this._rds$ = [];
            this._latestValue = false;
        };
        return AnyIsInProgressPipe;
    }());
    AnyIsInProgressPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'anyIsInProgress', pure: false },] }
    ];
    AnyIsInProgressPipe.ctorParameters = function () { return [
        { type: core.ChangeDetectorRef }
    ]; };
    var AnyIsNotAskedPipe = /** @class */ (function () {
        function AnyIsNotAskedPipe(cd) {
            this.cd = cd;
            this._latestValue = false;
            this._subscription = null;
            this._rds$ = [];
        }
        AnyIsNotAskedPipe.prototype.transform = function (rds$) {
            var _this = this;
            if (rds$ === null || rds$ === undefined) {
                return false;
            }
            if (this._rds$ === rds$) {
                return this._latestValue;
            }
            else if (this._subscription) {
                this.dispose();
            }
            this._rds$ = rds$;
            this._subscription = rxjs.combineLatest(rds$).subscribe(function (rds) {
                _this._latestValue = rds.some(isNotAsked);
                _this.cd.markForCheck();
            });
            return this._latestValue;
        };
        AnyIsNotAskedPipe.prototype.ngOnDestroy = function () {
            this.dispose();
        };
        AnyIsNotAskedPipe.prototype.dispose = function () {
            if (this._subscription) {
                this._subscription.unsubscribe();
            }
            this._rds$ = [];
            this._latestValue = false;
        };
        return AnyIsNotAskedPipe;
    }());
    AnyIsNotAskedPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'anyIsNotAsked', pure: false },] }
    ];
    AnyIsNotAskedPipe.ctorParameters = function () { return [
        { type: core.ChangeDetectorRef }
    ]; };
    var IsFailurePipe = /** @class */ (function () {
        function IsFailurePipe() {
        }
        IsFailurePipe.prototype.transform = function (rd) {
            if (rd === null || rd === undefined) {
                return false;
            }
            assertIsRemoteData(rd);
            return isFailure(rd);
        };
        return IsFailurePipe;
    }());
    IsFailurePipe.decorators = [
        { type: core.Pipe, args: [{ name: 'isFailure' },] }
    ];
    var IsSuccessPipe = /** @class */ (function () {
        function IsSuccessPipe() {
        }
        IsSuccessPipe.prototype.transform = function (rd) {
            if (rd === null || rd === undefined) {
                return false;
            }
            assertIsRemoteData(rd);
            return isSuccess(rd);
        };
        return IsSuccessPipe;
    }());
    IsSuccessPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'isSuccess' },] }
    ];
    var HasValuePipe = /** @class */ (function () {
        function HasValuePipe() {
        }
        HasValuePipe.prototype.transform = function (rd) {
            if (isSuccess(rd)) {
                return true;
            }
            else if ((isInProgress(rd) || isFailure(rd)) &&
                rd.value !== null &&
                rd.value !== undefined) {
                return true;
            }
            else if (rd === undefined || rd === null) {
                return false;
            }
            assertIsRemoteData(rd);
            return false;
        };
        return HasValuePipe;
    }());
    HasValuePipe.decorators = [
        { type: core.Pipe, args: [{ name: 'hasValue' },] }
    ];
    var GetSuccessPipe = /** @class */ (function () {
        function GetSuccessPipe() {
        }
        GetSuccessPipe.prototype.transform = function (rd, defaultValue) {
            if (isSuccess(rd)) {
                return rd.value;
            }
            else if (rd === undefined || rd === null) {
                return defaultValue;
            }
            assertIsRemoteData(rd);
            return defaultValue;
        };
        return GetSuccessPipe;
    }());
    GetSuccessPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'successValue' },] }
    ];
    var GetInProgressPipe = /** @class */ (function () {
        function GetInProgressPipe() {
        }
        GetInProgressPipe.prototype.transform = function (rd, defaultValue) {
            if (isInProgress(rd)) {
                return rd.value;
            }
            else if (rd === undefined || rd === null) {
                return defaultValue;
            }
            assertIsRemoteData(rd);
            return defaultValue;
        };
        return GetInProgressPipe;
    }());
    GetInProgressPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'inProgressValue' },] }
    ];
    var GetRemoteDataValuePipe = /** @class */ (function () {
        function GetRemoteDataValuePipe() {
        }
        GetRemoteDataValuePipe.prototype.transform = function (rd) {
            if (rd === undefined || rd === null) {
                return undefined;
            }
            assertIsRemoteData(rd);
            return isInProgress(rd) || isSuccess(rd) || isFailure(rd)
                ? rd.value
                : undefined;
        };
        return GetRemoteDataValuePipe;
    }());
    GetRemoteDataValuePipe.decorators = [
        { type: core.Pipe, args: [{ name: 'remoteDataValue' },] }
    ];
    var GetFailureErrorPipe = /** @class */ (function () {
        function GetFailureErrorPipe() {
        }
        GetFailureErrorPipe.prototype.transform = function (rd) {
            if (rd === undefined || rd === null) {
                return undefined;
            }
            assertIsRemoteData(rd);
            return isFailure(rd) ? rd.error : undefined;
        };
        return GetFailureErrorPipe;
    }());
    GetFailureErrorPipe.decorators = [
        { type: core.Pipe, args: [{ name: 'failureError' },] }
    ];
    var GetFailureValuePipe = /** @class */ (function () {
        function GetFailureValuePipe() {
        }
        GetFailureValuePipe.prototype.transform = function (rd) {
            if (rd === undefined || rd === null) {
                return undefined;
            }
            assertIsRemoteData(rd);
            return isFailure(rd) ? rd.value : undefined;
        };
        return GetFailureValuePipe;
    }());
    GetFailureValuePipe.decorators = [
        { type: core.Pipe, args: [{ name: 'failureValue' },] }
    ];

    var declarations = [
        IsNotAskedPipe,
        IsInProgressPipe,
        IsFailurePipe,
        IsSuccessPipe,
        HasValuePipe,
        GetSuccessPipe,
        GetFailureErrorPipe,
        GetFailureValuePipe,
        GetInProgressPipe,
        GetRemoteDataValuePipe,
        AnyIsInProgressPipe,
        AnyIsNotAskedPipe
    ];
    var RemoteDataModule = /** @class */ (function () {
        function RemoteDataModule() {
        }
        return RemoteDataModule;
    }());
    RemoteDataModule.decorators = [
        { type: core.NgModule, args: [{
                    declarations: declarations,
                    imports: [],
                    exports: declarations
                },] }
    ];

    /*
     * Public API Surface of lib
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.AnyIsInProgressPipe = AnyIsInProgressPipe;
    exports.AnyIsNotAskedPipe = AnyIsNotAskedPipe;
    exports.GetFailureErrorPipe = GetFailureErrorPipe;
    exports.GetFailureValuePipe = GetFailureValuePipe;
    exports.GetInProgressPipe = GetInProgressPipe;
    exports.GetRemoteDataValuePipe = GetRemoteDataValuePipe;
    exports.GetSuccessPipe = GetSuccessPipe;
    exports.HasValuePipe = HasValuePipe;
    exports.IsFailurePipe = IsFailurePipe;
    exports.IsInProgressPipe = IsInProgressPipe;
    exports.IsNotAskedPipe = IsNotAskedPipe;
    exports.IsSuccessPipe = IsSuccessPipe;
    exports.RemoteDataModule = RemoteDataModule;
    exports.chain = chain;
    exports.failure = failure;
    exports.filterFailure = filterFailure;
    exports.filterSuccess = filterSuccess;
    exports.fold = fold;
    exports.getOrElse = getOrElse;
    exports.inProgress = inProgress;
    exports.isFailure = isFailure;
    exports.isInProgress = isInProgress;
    exports.isNotAsked = isNotAsked;
    exports.isRemoteData = isRemoteData;
    exports.isSuccess = isSuccess;
    exports.map = map;
    exports.mapFailure = mapFailure;
    exports.notAsked = notAsked;
    exports.success = success;
    exports.ɵ0 = ɵ0;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=ngx-remotedata.umd.js.map
